<?php

use CambioReal\CambioReal;

/**
 * Bank Transfer Payment Gateway.
 *
 * Provides a Bank Transfer Payment Gateway. Based on code by Mike Pepper.
 *
 * @version     1.0.0
 * @package     WooCommerce/Classes/Payment
 * @author      CambioReal
 */
class WC_Gateway_Pix_CambioReal extends WC_Abstract_Gateway_CambioReal
{
    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        $this->id = 'wc-cambioreal-pix';
        $this->method_title = __('CambioReal - Pix', 'woocommerce');
        $this->method_description = __('Cambio transparent PIX.', 'woocommerce');

        parent::__construct();

        add_action('woocommerce_thankyou_' . $this->id, [$this, 'thank_you_page']);
        add_action('woocommerce_view_order', [$this, 'view_order_page']);

        if ($this->enabled === 'yes') {
            add_action('admin_notices', [$this, 'checkRequiredPixFields']);
        }
    }

    /**
     * Get gateway icon.
     * @return string
     */
    public function get_icon()
    {
        $icon_html = sprintf('<img src="%s" alt="Pix" title="Pix">', esc_url(plugins_url('assets/img/pix.svg', WC_CAMBIOREAL_ROOT_FILE)));

        return apply_filters('woocommerce_gateway_icon', $icon_html, $this->id);
    }

	/**
	 * Return the gateway's description.
	 *
	 * @return string
	 */
	public function get_description()
    {
        $description_html = '<p>';

        if ($simulation = $this->simulate()) {
            $description_html .= __('Taxa de câmbio', 'wc-gateway-cambioreal') . ': <strong>R$ ' . number_format($simulation->rate, 4) . '</strong>';
            $description_html .= '<br>' . __('Total a pagar com IOF', 'wc-gateway-cambioreal') . ' (' . $simulation->iof_percentage . '%): <strong>R$ ' . (($simulation->currency === 'BRL') ? number_format($simulation->amount, 2) : number_format($simulation->result, 2)) . '</strong>';
        }

        $description_html .= '</p>';

		return apply_filters( 'woocommerce_gateway_description', $description_html, $this->id );
	}

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields()
    {
        $this->form_fields = [
            // Configurações de integração.
            'integration'   => [
                'title'       => __('Integration', 'wc-gateway-cambioreal'),
                'description' => sprintf(__('Enter your CambioReal API credentials. You can obtain the credentials in the direct API integration section, access the CambioReal panel by clicking <a href="%s" target="_blank">here</a>.', 'wc-gateway-cambioreal'), 'https://business.cambioreal.com/en/panel/integration'),
                'type'        => 'title',
            ],
            'enabled'       => [
                'title'   => __('Enable/Disable', 'woocommerce'),
                'label'   => __('Enable CambioReal Pix', 'woocommerce'),
                'type'    => 'checkbox',
                'default' => 'no',
            ],
            'app-id'        => [
                'title'       => __('App Id', 'wc-gateway-cambioreal'),
                'description' => __('To get this information go to the CambioReal website and enter your Company Panel > Integration > Magento', 'wc-gateway-cambioreal'),
                'default'     => '',
                'type'        => 'text',
                'desc_tip'    => true,
            ],
            'app-secret'    => [
                'title'       => __('App Secret', 'wc-gateway-cambioreal'),
                'description' => __('You will receive this information along with the App Id.', 'wc-gateway-cambioreal'),
                'default'     => '',
                'type'        => 'password',
                'desc_tip'    => true,
            ],
            'test-mode'     => [
                'title'       => __('CambioReal Sandbox', 'wc-gateway-cambioreal'),
                'label'       => __('Enable CambioReal sandbox', 'wc-gateway-cambioreal'),
                'description' => sprintf(__('CambioReal sandbox can be used to test payments. Sign up for a company account <a href="%s" target="_blank">here</a>.', 'wc-gateway-cambioreal'), 'https://sandbox.cambioreal.com/br/receber'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
            'debug-enabled' => [
                'title'       => __('Debug Log', 'wc-gateway-cambioreal'),
                'label'       => __('Enable Debug Log', 'wc-gateway-cambioreal'),
                'description' => __('Record all errors that occur when executing a transaction.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],

            // Configurações de visualização.
            'method-details' => [
                'title'       => __('Payment method details', 'wc-gateway-cambioreal'),
                'description' => __('Customization of how the payment method name is displayed.', 'wc-gateway-cambioreal'),
                'type'        => 'title',
            ],
            'title'          => [
                'title'       => __('Title', 'woocommerce'),
                'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                'default'     => __('CambioReal Pix', 'woocommerce'),
                'type'        => 'text',
                'desc_tip'    => true,
            ],

            // Configurações do método de pagamento.
            'advanced'   => [
                'title'       => __('Payment Options', 'wc-gateway-cambioreal'),
                'description' => __('Advanced payment method options.', 'wc-gateway-cambioreal'),
                'type'        => 'title',
            ],
            'duplicate'  => [
                'title'       => __('Duplicate', 'wc-gateway-cambioreal'),
                'label'       => __('Allow to generate another transaction after expiration', 'wc-gateway-cambioreal'),
                'description' => __('Allow to generate another transaction after expiration without the customer having to create another order in the system.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
            'take-rates' => [
                'title'       => __('Take Rates', 'wc-gateway-cambioreal'),
                'label'       => __('I want to be responsible for all fees', 'wc-gateway-cambioreal'),
                'description' => __('No fees will be charged from the customer (includes iof). They will be discounted from the business final payment.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
            'show-rate'  => [
                'title'       => __('Show cart simulation', 'wc-gateway-cambioreal'),
                'label'       => __('Enable cart simulation', 'wc-gateway-cambioreal'),
                'description' => __('This option shows in the cart the exchange rate and the value with IOF to be paid by the customer at the checkout of CambioReal.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
        ];
    }

    function checkRequiredPixFields()
    {
        if (! function_exists('WC')) {
            return;
        }

        $missingFields = [];
        $requiredFields = [
            'billing_cpf',
            'billing_cnpj',
            'billing_first_name',
            'billing_last_name',
            'billing_birthdate',
        ];
        $requiredFields[] = 'billing_address_1';
        $requiredFields[] = 'billing_number';
        $requiredFields[] = 'billing_address_2';
        $requiredFields[] = 'billing_neighborhood';
        $requiredFields[] = 'billing_country';
        $requiredFields[] = 'billing_city';
        $requiredFields[] = 'billing_state';
        $requiredFields[] = 'billing_postcode';
        $checkoutFields = WC()->countries->get_address_fields(WC()->countries->get_base_country());

        foreach ($requiredFields as $field) {
            if (!array_key_exists($field, $checkoutFields)) {
                $missingFields[] = $field;
            }
        }

        if ((in_array('billing_cpf', $missingFields) && !in_array('billing_cnpj', $missingFields)) ||
            (in_array('billing_cnpj', $missingFields) && !in_array('billing_cpf', $missingFields))
        ) {
            array_shift($missingFields);
        }

        if (empty($missingFields)) {
            return;
        }

        $message = __('The following checkout fields are required for pix payment, but were not found:', 'wc-cambioreal');
        $message .= '</p><ul>';

        foreach ($missingFields as $field) {
            $message .= "<li>{$field}</li>";
        }

        $message .= '</ul><p>';
        $message .= __('Please, make sure to include them for CambioReal module to work.', 'wc-cambioreal');
        $message .= '</p><p>';
        $message .= sprintf(
            __('You can install %s or any other plugin of your choice to add the missing fields. %sRead documentation »%s', 'wc-cambioreal'),
            '<a href="https://wordpress.org/plugins/woocommerce-extra-checkout-fields-for-brazil" target="_blank" rel="noopener">Brazilian Market on WooCommerce</a>',
            '<a href="https://developer.cambioreal.com/#woocommerce" target="_blank" rel="noopener">',
            '</a>'
        );

        wcRenderAdminNoticeHtml($message);
    }

    /**
     * Process the payment and return the result.
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);

        $items = $order->get_items();
        $products = [];
        $total = 0;

        // Products
        foreach ($items as $item) {
            $product = wc_get_product($item['product_id']);
            $terms = wp_get_post_terms($item['product_id'], 'product_cat', ['fields' => 'names']);
            $brands = wp_get_post_terms($item['product_id'], 'product_brand', ['fields' => 'names']);

            if (empty($brands) || $brands instanceof WP_Error) {
                $brands = $product->get_attribute('brand') ?: $product->get_attribute('marca');
                $brands = array_filter([$brands]);
            }

            $products[] = [
                'descricao'  => $product->get_title(),
                'ref'        => $product->get_id(),
                'base_value' => $item['line_subtotal'],
                'valor'      => $item['line_total'],
                'qty'        => $item['qty'],
                'category'   => implode(',', $terms),
                'brand'      => implode(',', $brands),
                'sku'        => $product->get_sku(),
            ];

            $total += $item['line_total'];
        }

        // Shipping Tax
        if ($shipping = $order->get_shipping_total()) {
            $products[] = [
                'descricao'  => $order->get_shipping_method(),
                'ref'        => 'shipping',
                'base_value' => $shipping,
                'valor'      => $shipping,
            ];

            $total += $shipping;
        }

        // Fix to adjust cart price when users use a custom discount plugin for WooCommerce.
        if ($total != $order->get_total()) {
            $adjustment = $order->get_total() - $total;

            $products[] = [
                'descricao'  => 'WooCommerce Total Adjustment',
                'ref'        => 'adjustment',
                'base_value' => $adjustment,
                'valor'      => $adjustment,
            ];
        }

        $billingData = $order->get_data()['billing'];
        $document = $order->get_meta('_billing_cpf');

        if (empty($document)) {
            $document = $order->get_meta('_billing_cnpj');
        }

        $params = [
            'client' => [
                'name'       => $order->get_formatted_billing_full_name(),
                'email'      => $billingData['email'],
                'document'   => $document,
                'birth_date' => $order->get_meta('_billing_birthdate'),
                'phone'      => $order->get_meta('_billing_cellphone') ?: $billingData['phone'],
                'address' => [
                    'state'    => $billingData['state'],
                    'city'     => $billingData['city'],
                    'zip_code' => $billingData['postcode'],
                    'district' => $order->get_meta('_billing_neighborhood'),
                    'street'   => $billingData['address_1'],
                    'number'   => $order->get_meta('_billing_number'),
                ],
                'ip' => $order->get_customer_ip_address(),
            ],
            'currency'       => get_woocommerce_currency(),
            'amount'         => $order->get_total(),
            'payment_method' => 'pix',
            'order_id'       => $order_id,
            'duplicate'      => $this->duplicate,
            'take_rates'     => $this->take_rates,
            'products'       => $products,
        ];

        WC_CambioReal::log(sprintf(__('Request: %s', 'wc-gateway-cambioreal'), print_r($params, true)), $this);

        try {
            // Create solicitation
            $response = CambioReal::requestV2($params);

            return $this->process_response($response, $order);
        } catch (Exception $e) {
            WC_CambioReal::log($e->getMessage(), $this);
        }

        $message = $e->getMessage();

        if ($message) {
            wc_add_notice("Oops! Ocorreu um erro: $message.", 'error');
        } else {
            wc_add_notice("Oops! Ocorreu um erro, entre em contato conosco ou tente novamente mais tarde.", 'error');
        }

        if (isset($response->errors)) {
            foreach ($response->errors as $error) {
                wc_add_notice($error, 'error');
            }
        }
    }

    /**
     *
     * @param StdClass $response
     * @param WC_Order $order
     *
     * @throws Exception Throws error message.
     * @throws Exception Throws parameter missing exception.
     */
    protected function process_response($response, $order)
    {
        WC_CambioReal::log(sprintf(__('Processing response: %s', 'wc-gateway-cambioreal'), print_r($response, true)), $this);

        if ($response->status !== 'success') {
            $this->process_response_error($response, $order);
        }

        $message = sprintf(__('Transaction created. Token: %s', 'wc-gateway-cambioreal'), $response->data->token);

        WC_CambioReal::log($message, $this);

        // Mark as on-hold (we're awaiting the payment)
        // $order->update_status('on-hold', __('Awaiting CambioReal payment.', 'woocommerce'));

        // Save the cart
        update_user_meta(get_current_user_id(), '_cambioreal_persistent_cart', WC()->cart->get_cart());

        // Reduce stock levels
        wc_reduce_stock_levels($order->get_id());

        // Empty cart
        WC()->cart->empty_cart();

        $order->add_order_note(__("CambioReal: The request {$response->data->code} was created.", 'wc-gateway-cambioreal'));

        // Save post's meta fields.
        $this->save_order_meta_fields($order, $response->data);

        return [
            'result'   => 'success',
            'redirect' => $this->get_return_url($order)
        ];
    }

    /**
     *
     * @param StdClass $response
     * @param WC_Order $order
     *
     * @throws Exception Throws error message.
     * @throws Exception Throws parameter missing exception.
     */
    protected function process_response_error($response, $order)
    {
        $error_message = sprintf(__('CambioReal: An error occurred: %s', 'wc-gateway-cambioreal'), $response->message);

        $order->update_status('failed', $error_message);
        $order->add_order_note($error_message);

        throw new Exception($response->message);
    }

    /**
     * Save order's meta fields for future use
     *
     * @param  WC_Order $order The order created.
     * @param  Object   $data The data from CambioReal success response.
     * @return void
     */
    protected function save_order_meta_fields($order, $data)
    {
        update_post_meta($order->get_id(), '_cr_token', $data->id);
        update_post_meta($order->get_id(), '_cr_ticket', $data->transaction->ticket_url);
        update_post_meta($order->get_id(), '_cr_pix', $data->transaction);
    }

    /**
     * @param int $orderId
     * @return void
     */
    public function view_order_page($orderId)
    {
        $this->thank_you_page($orderId);
    }

    /**
     * @param int $orderId
     * @return void
     * @throws Exception
     */
    public function thank_you_page($orderId)
    {
        $order = wc_get_order($orderId);
        $transaction = get_post_meta($order->get_id(), '_cr_pix', true);

        ?>
        <?php if (is_account_page()) : ?>
            <h2><?= __('Payment details', 'wc-gateway-cambioreal'); ?></h2>
        <?php endif; ?>
        <div class="woocommerce-<?= $order->get_status() !== 'failed' ? 'message' : 'error' ?>">
            <div class="cambioreal-order-response">

                <?php if ($order->get_status() === 'failed') : ?>
                    <p><?= __('Unfortunately, your transaction has not been processed.', 'wc-gateway-cambioreal') ?></p>
                    <p><?= __('Please try again or contact us for more information.', 'wc-gateway-cambioreal') ?></p>
                <?php elseif (in_array($order->get_status(), ['completed', 'processing'])) : ?>
                    <p>
                        <?php
                            printf(
                                __('The status of your transaction is %s.', 'wc-gateway-cambioreal'),
                                '<strong>' . strtoupper($order->get_status()) . '</strong>'
                            );
                        ?>
                    </p>
                <?php else : ?>
                    <?php if (! is_account_page()) : ?>
                        <p class="title"><?= __('Your transaction has been processed.', 'wc-gateway-cambioreal') ?></p>
                    <?php endif; ?>
                    <p><img src="<?= $transaction->barcode ?>" alt="Pix QRCode" class="cambioreal-qr-code-img" /></p>
                    <button type="button"
                            id="cambioreal-qr-code-button"
                            data-raw-code="<?= $transaction->number ?>"
                            class="cambioreal-payment-button">
                        <?= __('Copy Code', 'wc-gateway-cambioreal'); ?>
                    </button>
                    <div class="cambioreal-flex-container">
                        <?php foreach ($this->getInstructions() as $instruction) : ?>
                            <div class="cambioreal-qr-code-instruction">
                                <p><?= __($instruction, 'wc-gateway-cambioreal'); ?></p>
                            </div>
                        <?php endforeach; ?>
                        <div class="cambioreal-qr-code-instruction cambioreal-qr-code-attention">
                            <p>
                                <span>
                                    <svg xmlns="http://www.w3.org/2000/svg" width="15" height="15" viewBox="0 0 15 15" fill="none"><path d="M7.5 0C6.01664 0 4.56659 0.439867 3.33323 1.26398C2.09986 2.08809 1.13856 3.25943 0.570907 4.62987C0.00324963 6.00032 -0.145275 7.50832 0.144114 8.96317C0.433503 10.418 1.14781 11.7544 2.1967 12.8033C3.2456 13.8522 4.58197 14.5665 6.03683 14.8559C7.49168 15.1453 8.99968 14.9967 10.3701 14.4291C11.7406 13.8614 12.9119 12.9001 13.736 11.6668C14.5601 10.4334 15 8.98336 15 7.5C14.9978 5.51155 14.2069 3.60518 12.8009 2.19914C11.3948 0.793098 9.48845 0.00221143 7.5 0ZM7.5 13.5C6.31331 13.5 5.15328 13.1481 4.16658 12.4888C3.17989 11.8295 2.41085 10.8925 1.95673 9.7961C1.5026 8.69974 1.38378 7.49334 1.61529 6.32946C1.8468 5.16557 2.41825 4.09647 3.25736 3.25736C4.09648 2.41824 5.16558 1.8468 6.32946 1.61529C7.49335 1.38378 8.69975 1.5026 9.7961 1.95672C10.8925 2.41085 11.8295 3.17988 12.4888 4.16658C13.1481 5.15327 13.5 6.31331 13.5 7.5C13.4982 9.09074 12.8655 10.6158 11.7406 11.7406C10.6158 12.8654 9.09074 13.4982 7.5 13.5ZM7.5 7.125C7.30109 7.125 7.11032 7.20402 6.96967 7.34467C6.82902 7.48532 6.75 7.67608 6.75 7.875V10.125C6.75 10.3239 6.82902 10.5147 6.96967 10.6553C7.11032 10.796 7.30109 10.875 7.5 10.875C7.69891 10.875 7.88968 10.796 8.03033 10.6553C8.17098 10.5147 8.25 10.3239 8.25 10.125V7.875C8.25 7.67608 8.17098 7.48532 8.03033 7.34467C7.88968 7.20402 7.69891 7.125 7.5 7.125ZM7.5 4.125C7.31458 4.125 7.13333 4.17998 6.97916 4.283C6.82498 4.38601 6.70482 4.53243 6.63387 4.70373C6.56291 4.87504 6.54434 5.06354 6.58052 5.2454C6.61669 5.42725 6.70598 5.5943 6.83709 5.72541C6.9682 5.85652 7.13525 5.94581 7.31711 5.98198C7.49896 6.01816 7.68746 5.99959 7.85877 5.92864C8.03007 5.85768 8.17649 5.73752 8.2795 5.58335C8.38252 5.42917 8.4375 5.24792 8.4375 5.0625C8.4375 4.81386 8.33873 4.5754 8.16292 4.39959C7.9871 4.22377 7.74864 4.125 7.5 4.125Z" fill="white"/></svg>
                                </span>
                                <?php
                                _e(
                                    'You can also complete the payment by copying and pasting the code into the app.',
                                    'wc-gateway-cambioreal'
                                );
                                ?>
                            </p>
                        </div>
                    </div>
                    <p><?= __('If you have any questions regarding the transaction, please contact us.', 'wc-gateway-cambioreal') ?></p>
                <?php endif; ?>
            </div>
        </div>
        <?php
    }

    /**
     * @return array
     */
    public function getInstructions()
    {
        return [
            '<span>1.</span> Point your phone at this screen to capture the code.',
            '<span>2.</span> Open your payments app.',
            '<span>3.</span> Confirm the information and complete the payment on the app.',
            '<span>4.</span> We will send you a purchase confirmation.'
        ];
    }
}
